unit adxOLRemoveCmdBars_IMPL;

interface

uses
  SysUtils, ComObj, ComServ, ActiveX, Variants, adxAddIn, adxOLRemoveCmdBars_TLB, StdVcl,
  Classes, Outlook2000, Office2000;

type
  TADX_OLRemoveCmdBars = class(TadxAddin, IADX_OLRemoveCmdBars)
  end;

  TAddInModule = class(TadxCOMAddInModule)
    TestExplorerCommandbar: TadxOlExplorerCommandbar;
    TestInspectorCommandBar: TadxOlInspectorCommandBar;
    procedure adxCOMAddInModuleBeforeAddinUnregister(Sender: TObject);
  private
    procedure RemoveOutlookExplorerCommandBars(const IOutlook: _Application;
      defFolders: OlDefaultFolders);
    procedure RemoveOutlookInspectorCommandBars(const IOutlook: _Application;
      itemType: OlItemType);
    procedure UninstallCommandBar(const ICmdBars: CommandBars;
      const CmdBar: TadxCommandBar);
  protected
  public
  end;

implementation

{$R *.dfm}

function GetActiveOleObject2(const ClassName: string): IDispatch;
var
  ClassID: TCLSID;
  IUnk: IUnknown;
begin
  Result := nil;
  ClassID := ProgIDToClassID(ClassName);
  GetActiveObject(ClassID, nil, IUnk);
  if Assigned(IUnk) then
    IUnk.QueryInterface(IDispatch, Result);
end;

function CreateOleObject2(const ClassName: string): IDispatch;
var
  ClassID: TCLSID;
begin
  Result := nil;
  ClassID := ProgIDToClassID(ClassName);
  CoCreateInstance(ClassID, nil, CLSCTX_INPROC_SERVER or CLSCTX_LOCAL_SERVER, IDispatch, Result);
end;

procedure TAddInModule.adxCOMAddInModuleBeforeAddinUnregister(Sender: TObject);
var
  i: Integer;
  NeedClose: boolean;
  isExplorerTemporary,
  isInspectorTemporary: boolean;
  FOutlook: Outlook2000._Application;
begin
  if HostType <> ohaOutlook then Exit;
  isExplorerTemporary := True;
  isInspectorTemporary := True;
  for i := 0 to CommandBarsCount - 1 do
    if not CommandBars[i].Temporary and (ohaOutlook in CommandBars[i].SupportedApps) then begin
      if CommandBars[i] is TadxOLInspectorCommandBar then
        isInspectorTemporary := False
      else
        isExplorerTemporary := False;
    end;

  if not isExplorerTemporary or not isInspectorTemporary then begin
    NeedClose := False;
    FOutlook := GetActiveOleObject2('Outlook.Application') as Outlook2000._Application;
    if not Assigned(FOutlook) then begin
      FOutlook := CreateOleObject2('Outlook.Application') as Outlook2000._Application;
      NeedClose := True;
    end;
    if Assigned(FOutlook) then
      try
        if not isExplorerTemporary then begin
          try
            RemoveOutlookExplorerCommandBars(FOutlook, olFolderInbox);
          except
          end;
          try
            RemoveOutlookExplorerCommandBars(FOutlook, olFolderContacts);
          except
          end;
          try
            RemoveOutlookExplorerCommandBars(FOutlook, olFolderCalendar);
          except
          end;
          try
           RemoveOutlookExplorerCommandBars(FOutlook, olFolderTasks);
          except
          end;
        end;
        if not isInspectorTemporary then begin
          try
            RemoveOutlookInspectorCommandBars(FOutlook, olAppointmentItem);
          except
          end;
          try
            RemoveOutlookInspectorCommandBars(FOutlook, olContactItem);
          except
          end;
          try
            RemoveOutlookInspectorCommandBars(FOutlook, olDistributionListItem);
          except
          end;
          try
            RemoveOutlookInspectorCommandBars(FOutlook, olJournalItem);
          except
          end;
          try
            RemoveOutlookInspectorCommandBars(FOutlook, olMailItem);
          except
          end;
          try
            RemoveOutlookInspectorCommandBars(FOutlook, olPostItem);
          except
          end;
          try
            RemoveOutlookInspectorCommandBars(FOutlook, olTaskItem);
          except
          end;
        end;
      finally
        if NeedClose then FOutlook.Quit;
        FOutlook := nil;
      end;
  end;
end;

procedure TAddInModule.RemoveOutlookExplorerCommandBars(const IOutlook: Outlook2000._Application;
  defFolders: OlDefaultFolders);
var
  i: Integer;
  INameSpace: _Namespace;
  IFolder: MAPIFolder;
  IExplorer: _Explorer;
  ICommandBars: _CommandBars;
begin
  INameSpace := IOutlook.GetNamespace('MAPI');
  IFolder := INameSpace.GetDefaultFolder(defFolders);
  IExplorer := IFolder.GetExplorer(EmptyParam);
  ICommandBars := IExplorer.CommandBars;
  if Assigned(ICommandBars) then begin
    for i := 0 to CommandBarsCount - 1 do
      if not CommandBars[i].Temporary and (ohaOutlook in CommandBars[i].SupportedApps) then
        if not (CommandBars[i] is TadxOLInspectorCommandBar) then
          UninstallCommandBar(ICommandBars, CommandBars[i]);
  end;
  ICommandBars := nil;
  if Assigned(IExplorer) then IExplorer.Close;
  IExplorer := nil;
  IFolder := nil;
  INameSpace := nil;
end;

procedure TAddInModule.RemoveOutlookInspectorCommandBars(const IOutlook: Outlook2000._Application;
  itemType: OlItemType);
var
  i: Integer;
  item: OleVariant;
  IInspector: _Inspector;
  ICommandBars: _CommandBars;
begin
  item := IOutlook.CreateItem(itemType);
  IInspector := IDispatch(item.GetInspector) as Outlook2000._Inspector;
  ICommandBars := IInspector.CommandBars;
  if Assigned(ICommandBars) then begin
    for i := 0 to CommandBarsCount - 1 do
      if not CommandBars[i].Temporary and (ohaOutlook in CommandBars[i].SupportedApps) then
        if CommandBars[i] is TadxOLInspectorCommandBar then
          UninstallCommandBar(ICommandBars, CommandBars[i]);
  end;
  ICommandBars := nil;
  if Assigned(IInspector) then IInspector.Close(olDiscard);
  IInspector := nil;
  item := Unassigned;
end;

procedure TAddInModule.UninstallCommandBar(const ICmdBars: CommandBars;
  const CmdBar: TadxCommandBar);

  function FindCommandBar(const CmdBarName: WideString): CommandBar;
  begin
    Result := nil;
    if Assigned(ICmdBars) then
      try
        Result := ICmdBars.Get_Item(CmdBarName);
      except
        Result := nil;
      end;
  end;

  procedure DeletePopupControls(ACmdBar: CommandBar; AControls: TadxCommandBarControls);
  var
    i: Integer;
    ICtrl: CommandBarControl;
  begin
    for i := 0 to AControls.Count - 1 do begin
      ICtrl := ACmdBar.FindControl(EmptyParam, EmptyParam, AControls[i].OfficeTag, EmptyParam, True);
      if Assigned(ICtrl) then
        try
          if CmdBar.Controls[i] is TadxCommandBarPopup then
            DeletePopupControls(ACmdBar, AControls[i].AsPopup.Controls);
          if not ICtrl.BuiltIn then ICtrl.Delete(False);
        finally
          ICtrl := nil;
        end;
    end;
  end;

var
  i: Integer;
  ICmdBar: CommandBar;
  ICtrl: CommandBarControl;
begin
  ICmdBar := FindCommandBar(CmdBar.CommandBarName);
  if Assigned(ICmdBar) then
    try
      for i := 0 to CmdBar.Controls.Count - 1 do begin
        ICtrl := ICmdBar.FindControl(EmptyParam, EmptyParam, CmdBar.Controls[i].OfficeTag, EmptyParam, True);
        if Assigned(ICtrl) then
          try
            if CmdBar.Controls[i] is TadxCommandBarPopup then
              DeletePopupControls(ICmdBar, CmdBar.Controls[i].AsPopup.Controls);
            if not ICtrl.BuiltIn then ICtrl.Delete(False);
          finally
            ICtrl := nil;
          end;
      end;
      if not ICmdBar.BuiltIn then ICmdBar.Delete;
    finally
      ICmdBar := nil;
    end;
end;

initialization
  TadxFactory.Create(ComServer, TADX_OLRemoveCmdBars, CLASS_ADX_OLRemoveCmdBars, TAddInModule);

end.
